// This file is autogenerated. Changes will be overwritten.
#include "IJHAR2IVER/moc_clickablelabel.cpp"
#include "IJHAR2IVER/moc_colorpickereditmode.cpp"
#include "IJHAR2IVER/moc_colorpickereditor.cpp"
#include "IJHAR2IVER/moc_configwindow.cpp"
#include "IJHAR2IVER/moc_extendedslider.cpp"
#include "IJHAR2IVER/moc_filenameeditor.cpp"
#include "IJHAR2IVER/moc_generalconf.cpp"
#include "IJHAR2IVER/moc_setshortcutwidget.cpp"
#include "IJHAR2IVER/moc_shortcutswidget.cpp"
#include "IJHAR2IVER/moc_strftimechooserwidget.cpp"
#include "IJHAR2IVER/moc_styleoverride.cpp"
#include "IJHAR2IVER/moc_uicoloreditor.cpp"
#include "IJHAR2IVER/moc_visualseditor.cpp"
#include "TAC5DWH4SE/moc_flameshot.cpp"
#include "TAC5DWH4SE/moc_flameshotdaemon.cpp"
#include "TAC5DWH4SE/moc_flameshotdbusadapter.cpp"
#include "CB6E7VNMGS/moc_abstractactiontool.cpp"
#include "CB6E7VNMGS/moc_abstractpathtool.cpp"
#include "CB6E7VNMGS/moc_abstracttwopointtool.cpp"
#include "EPZ2JSH3OC/moc_accepttool.cpp"
#include "EBZTUYDCZR/moc_arrowtool.cpp"
#include "CB6E7VNMGS/moc_capturetool.cpp"
#include "O6J2I75NMM/moc_circletool.cpp"
#include "SBONXDZUGY/moc_circlecounttool.cpp"
#include "OTWB2I47FB/moc_copytool.cpp"
#include "PWRGMN3A6Q/moc_exittool.cpp"
#include "BRMITS7247/moc_inverttool.cpp"
#include "IBPZ6RSZN2/moc_applaunchertool.cpp"
#include "IBPZ6RSZN2/moc_applauncherwidget.cpp"
#include "IBPZ6RSZN2/moc_launcheritemdelegate.cpp"
#include "IBPZ6RSZN2/moc_terminallauncher.cpp"
#include "D7BN5JOYNZ/moc_linetool.cpp"
#include "YLULWPRF4O/moc_markertool.cpp"
#include "PC6Y4SYLEZ/moc_movetool.cpp"
#include "KBB4SY62JM/moc_penciltool.cpp"
#include "PGJOYVEBXU/moc_pintool.cpp"
#include "PGJOYVEBXU/moc_pinwidget.cpp"
#include "D3TIQD6DJB/moc_pixelatetool.cpp"
#include "HAUO5V4B6Y/moc_rectangletool.cpp"
#include "ZY7JWYPUWB/moc_redotool.cpp"
#include "ZREUGHW3NY/moc_savetool.cpp"
#include "OQP4VDJJHG/moc_selectiontool.cpp"
#include "PYQL3SLD4E/moc_sizedecreasetool.cpp"
#include "LOCAFNBQSH/moc_sizeincreasetool.cpp"
#include "CJ6UZF23FQ/moc_textconfig.cpp"
#include "CJ6UZF23FQ/moc_texttool.cpp"
#include "CJ6UZF23FQ/moc_textwidget.cpp"
#include "CB6E7VNMGS/moc_toolfactory.cpp"
#include "BTVB2UQ274/moc_undotool.cpp"
#include "7HEHEGDA3T/moc_confighandler.cpp"
#include "7HEHEGDA3T/moc_filenamehandler.cpp"
#include "7HEHEGDA3T/moc_monitorpreview.cpp"
#include "7HEHEGDA3T/moc_request.cpp"
#include "7HEHEGDA3T/moc_screengrabber.cpp"
#include "7HEHEGDA3T/moc_systemnotification.cpp"
#include "3PDZ6QXSRB/moc_buttonhandler.cpp"
#include "3PDZ6QXSRB/moc_capturebutton.cpp"
#include "3PDZ6QXSRB/moc_capturetoolbutton.cpp"
#include "3PDZ6QXSRB/moc_capturewidget.cpp"
#include "3PDZ6QXSRB/moc_colorpicker.cpp"
#include "3PDZ6QXSRB/moc_hovereventfilter.cpp"
#include "3PDZ6QXSRB/moc_magnifierwidget.cpp"
#include "3PDZ6QXSRB/moc_notifierbox.cpp"
#include "3PDZ6QXSRB/moc_selectionwidget.cpp"
#include "IMAN36LHMA/moc_capturelauncher.cpp"
#include "IMAN36LHMA/moc_colorpickerwidget.cpp"
#include "IMAN36LHMA/moc_draggablewidgetmaker.cpp"
#include "IMAN36LHMA/moc_imagelabel.cpp"
#include "IMAN36LHMA/moc_infowindow.cpp"
#include "IMAN36LHMA/moc_loadspinner.cpp"
#include "IMAN36LHMA/moc_notificationwidget.cpp"
#include "IMAN36LHMA/moc_orientablepushbutton.cpp"
#include "P3BPLNLAOU/moc_colorgrabwidget.cpp"
#include "P3BPLNLAOU/moc_sidepanelwidget.cpp"
#include "P3BPLNLAOU/moc_utilitypanel.cpp"
#include "IMAN36LHMA/moc_trayicon.cpp"
