// SPDX-License-Identifier: GPL-3.0-or-later
// SPDX-FileCopyrightText: 2017-2019 Alejandro Sirgo Rica & Contributors

#pragma once

#include <QStringList>

namespace PathInfo { // namespace

const QString whiteIconPath();

const QString blackIconPath();

QStringList translationsPaths();

} // namespace
