/****************************************************************************
** Meta object code from reading C++ file 'flameshot.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../flameshot-14.0.0/src/core/flameshot.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'flameshot.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9FlameshotE_t {};
} // unnamed namespace

template <> constexpr inline auto Flameshot::qt_create_metaobjectdata<qt_meta_tag_ZN9FlameshotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Flameshot",
        "captureTaken",
        "",
        "QPixmap",
        "p",
        "captureFailed",
        "gui",
        "CaptureWidget*",
        "CaptureRequest",
        "req",
        "screen",
        "screenNumber",
        "full",
        "launcher",
        "config",
        "info",
        "openSavePath",
        "getVersion",
        "QVersionNumber",
        "requestCapture",
        "request",
        "exportCapture",
        "QRect&",
        "selection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'captureTaken'
        QtMocHelpers::SignalData<void(QPixmap)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'captureFailed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'gui'
        QtMocHelpers::SlotData<CaptureWidget *(const CaptureRequest &)>(6, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'gui'
        QtMocHelpers::SlotData<CaptureWidget *()>(6, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 7),
        // Slot 'screen'
        QtMocHelpers::SlotData<void(CaptureRequest, int const)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'screen'
        QtMocHelpers::SlotData<void(CaptureRequest)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'full'
        QtMocHelpers::SlotData<void(const CaptureRequest &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'launcher'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'config'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'info'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openSavePath'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getVersion'
        QtMocHelpers::SlotData<QVersionNumber()>(17, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'requestCapture'
        QtMocHelpers::SlotData<void(const CaptureRequest &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 20 },
        }}),
        // Slot 'exportCapture'
        QtMocHelpers::SlotData<void(const QPixmap &, QRect &, const CaptureRequest &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 22, 23 }, { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Flameshot, qt_meta_tag_ZN9FlameshotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Flameshot::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9FlameshotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9FlameshotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9FlameshotE_t>.metaTypes,
    nullptr
} };

void Flameshot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Flameshot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->captureTaken((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 1: _t->captureFailed(); break;
        case 2: { CaptureWidget* _r = _t->gui((*reinterpret_cast<std::add_pointer_t<CaptureRequest>>(_a[1])));
            if (_a[0]) *reinterpret_cast<CaptureWidget**>(_a[0]) = std::move(_r); }  break;
        case 3: { CaptureWidget* _r = _t->gui();
            if (_a[0]) *reinterpret_cast<CaptureWidget**>(_a[0]) = std::move(_r); }  break;
        case 4: _t->screen((*reinterpret_cast<std::add_pointer_t<CaptureRequest>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->screen((*reinterpret_cast<std::add_pointer_t<CaptureRequest>>(_a[1]))); break;
        case 6: _t->full((*reinterpret_cast<std::add_pointer_t<CaptureRequest>>(_a[1]))); break;
        case 7: _t->launcher(); break;
        case 8: _t->config(); break;
        case 9: _t->info(); break;
        case 10: _t->openSavePath(); break;
        case 11: { QVersionNumber _r = _t->getVersion();
            if (_a[0]) *reinterpret_cast<QVersionNumber*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->requestCapture((*reinterpret_cast<std::add_pointer_t<CaptureRequest>>(_a[1]))); break;
        case 13: _t->exportCapture((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRect&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CaptureRequest>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Flameshot::*)(QPixmap )>(_a, &Flameshot::captureTaken, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Flameshot::*)()>(_a, &Flameshot::captureFailed, 1))
            return;
    }
}

const QMetaObject *Flameshot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Flameshot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9FlameshotE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Flameshot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void Flameshot::captureTaken(QPixmap _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Flameshot::captureFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
