/********************************************************************************
** Form generated from reading UI file 'infowindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INFOWINDOW_H
#define UI_INFOWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_InfoWindow
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QLabel *IconSVG;
    QLabel *License;
    QLabel *LicenseDetails;
    QSpacerItem *verticalSpacer;
    QLabel *Version;
    QLabel *VersionDetails;
    QLabel *OperatingSystemDetails;
    QPushButton *CopyInfoButton;

    void setupUi(QWidget *InfoWindow)
    {
        if (InfoWindow->objectName().isEmpty())
            InfoWindow->setObjectName("InfoWindow");
        InfoWindow->resize(182, 262);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/img/app/flameshot.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        InfoWindow->setWindowIcon(icon);
        gridLayout = new QGridLayout(InfoWindow);
        gridLayout->setObjectName("gridLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        IconSVG = new QLabel(InfoWindow);
        IconSVG->setObjectName("IconSVG");
        IconSVG->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(IconSVG);

        License = new QLabel(InfoWindow);
        License->setObjectName("License");
        QFont font;
        font.setBold(true);
        font.setUnderline(true);
        License->setFont(font);
        License->setAlignment(Qt::AlignCenter);
        License->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(License);

        LicenseDetails = new QLabel(InfoWindow);
        LicenseDetails->setObjectName("LicenseDetails");
        LicenseDetails->setAlignment(Qt::AlignCenter);
        LicenseDetails->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(LicenseDetails);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        Version = new QLabel(InfoWindow);
        Version->setObjectName("Version");
        Version->setFont(font);
        Version->setAlignment(Qt::AlignCenter);
        Version->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(Version);

        VersionDetails = new QLabel(InfoWindow);
        VersionDetails->setObjectName("VersionDetails");
        VersionDetails->setAlignment(Qt::AlignCenter);
        VersionDetails->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(VersionDetails);

        OperatingSystemDetails = new QLabel(InfoWindow);
        OperatingSystemDetails->setObjectName("OperatingSystemDetails");
        OperatingSystemDetails->setAlignment(Qt::AlignCenter);
        OperatingSystemDetails->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(OperatingSystemDetails);

        CopyInfoButton = new QPushButton(InfoWindow);
        CopyInfoButton->setObjectName("CopyInfoButton");

        verticalLayout->addWidget(CopyInfoButton);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        retranslateUi(InfoWindow);

        QMetaObject::connectSlotsByName(InfoWindow);
    } // setupUi

    void retranslateUi(QWidget *InfoWindow)
    {
        InfoWindow->setWindowTitle(QCoreApplication::translate("InfoWindow", "About", nullptr));
        IconSVG->setText(QCoreApplication::translate("InfoWindow", "Icon", nullptr));
        License->setText(QCoreApplication::translate("InfoWindow", "License", nullptr));
        LicenseDetails->setText(QCoreApplication::translate("InfoWindow", "GPLv3+", nullptr));
        Version->setText(QCoreApplication::translate("InfoWindow", "Version", nullptr));
        VersionDetails->setText(QCoreApplication::translate("InfoWindow", "Flameshot v", nullptr));
        OperatingSystemDetails->setText(QCoreApplication::translate("InfoWindow", "OS Info", nullptr));
        CopyInfoButton->setText(QCoreApplication::translate("InfoWindow", "Copy Info", nullptr));
    } // retranslateUi

};

namespace Ui {
    class InfoWindow: public Ui_InfoWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_INFOWINDOW_H
