/********************************************************************************
** Form generated from reading UI file 'capturelauncher.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CAPTURELAUNCHER_H
#define UI_CAPTURELAUNCHER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_CaptureLauncher
{
public:
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout_2;
    QLabel *modeLabel;
    QGridLayout *gridLayout_3;
    QLabel *areaLabel;
    QComboBox *captureType;
    QLabel *delayLabel;
    QLabel *monitorLabel;
    QComboBox *monitorSelection;
    QLabel *sizeLabel;
    QSpinBox *delayTime;
    QHBoxLayout *horizontalLayout;
    QLineEdit *screenshotWidth;
    QLineEdit *screenshotHeight;
    QLineEdit *screenshotX;
    QLineEdit *screenshotY;
    QLabel *countdownLabel;
    QPushButton *launchButton;

    void setupUi(QDialog *CaptureLauncher)
    {
        if (CaptureLauncher->objectName().isEmpty())
            CaptureLauncher->setObjectName("CaptureLauncher");
        CaptureLauncher->resize(0, 0);
        horizontalLayout_4 = new QHBoxLayout(CaptureLauncher);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(4, 4, 4, 4);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(4);
        verticalLayout_2->setObjectName("verticalLayout_2");
        modeLabel = new QLabel(CaptureLauncher);
        modeLabel->setObjectName("modeLabel");
        QFont font;
        font.setBold(true);
        modeLabel->setFont(font);
        modeLabel->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignVCenter);
        modeLabel->setMargin(-1);

        verticalLayout_2->addWidget(modeLabel);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        areaLabel = new QLabel(CaptureLauncher);
        areaLabel->setObjectName("areaLabel");

        gridLayout_3->addWidget(areaLabel, 1, 0, 1, 1);

        captureType = new QComboBox(CaptureLauncher);
        captureType->setObjectName("captureType");
        captureType->setSizeAdjustPolicy(QComboBox::SizeAdjustPolicy::AdjustToContents);

        gridLayout_3->addWidget(captureType, 1, 1, 1, 1);

        delayLabel = new QLabel(CaptureLauncher);
        delayLabel->setObjectName("delayLabel");

        gridLayout_3->addWidget(delayLabel, 3, 0, 1, 1);

        monitorLabel = new QLabel(CaptureLauncher);
        monitorLabel->setObjectName("monitorLabel");

        gridLayout_3->addWidget(monitorLabel, 2, 0, 1, 1);

        monitorSelection = new QComboBox(CaptureLauncher);
        monitorSelection->setObjectName("monitorSelection");

        gridLayout_3->addWidget(monitorSelection, 2, 1, 1, 1);

        sizeLabel = new QLabel(CaptureLauncher);
        sizeLabel->setObjectName("sizeLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(sizeLabel->sizePolicy().hasHeightForWidth());
        sizeLabel->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(sizeLabel, 4, 0, 1, 1);

        delayTime = new QSpinBox(CaptureLauncher);
        delayTime->setObjectName("delayTime");
        delayTime->setMinimum(0);
        delayTime->setMaximum(999);

        gridLayout_3->addWidget(delayTime, 3, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        screenshotWidth = new QLineEdit(CaptureLauncher);
        screenshotWidth->setObjectName("screenshotWidth");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(screenshotWidth->sizePolicy().hasHeightForWidth());
        screenshotWidth->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(screenshotWidth);

        screenshotHeight = new QLineEdit(CaptureLauncher);
        screenshotHeight->setObjectName("screenshotHeight");
        sizePolicy1.setHeightForWidth(screenshotHeight->sizePolicy().hasHeightForWidth());
        screenshotHeight->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(screenshotHeight);

        screenshotX = new QLineEdit(CaptureLauncher);
        screenshotX->setObjectName("screenshotX");
        sizePolicy1.setHeightForWidth(screenshotX->sizePolicy().hasHeightForWidth());
        screenshotX->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(screenshotX);

        screenshotY = new QLineEdit(CaptureLauncher);
        screenshotY->setObjectName("screenshotY");
        sizePolicy1.setHeightForWidth(screenshotY->sizePolicy().hasHeightForWidth());
        screenshotY->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(screenshotY);


        gridLayout_3->addLayout(horizontalLayout, 4, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout_3);

        countdownLabel = new QLabel(CaptureLauncher);
        countdownLabel->setObjectName("countdownLabel");

        verticalLayout_2->addWidget(countdownLabel, 0, Qt::AlignmentFlag::AlignHCenter);

        launchButton = new QPushButton(CaptureLauncher);
        launchButton->setObjectName("launchButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(launchButton->sizePolicy().hasHeightForWidth());
        launchButton->setSizePolicy(sizePolicy2);

        verticalLayout_2->addWidget(launchButton, 0, Qt::AlignmentFlag::AlignHCenter);


        horizontalLayout_4->addLayout(verticalLayout_2);


        retranslateUi(CaptureLauncher);

        QMetaObject::connectSlotsByName(CaptureLauncher);
    } // setupUi

    void retranslateUi(QDialog *CaptureLauncher)
    {
        CaptureLauncher->setWindowTitle(QCoreApplication::translate("CaptureLauncher", "Capture Launcher", nullptr));
        modeLabel->setText(QCoreApplication::translate("CaptureLauncher", "Capture Mode", nullptr));
        areaLabel->setText(QCoreApplication::translate("CaptureLauncher", "Area:", nullptr));
        captureType->setCurrentText(QString());
        delayLabel->setText(QCoreApplication::translate("CaptureLauncher", "Delay:", nullptr));
        monitorLabel->setText(QCoreApplication::translate("CaptureLauncher", "Monitor:", nullptr));
        sizeLabel->setText(QCoreApplication::translate("CaptureLauncher", "WxH+x+y", nullptr));
        delayTime->setSuffix(QCoreApplication::translate("CaptureLauncher", " seconds", nullptr));
        countdownLabel->setText(QString());
        launchButton->setText(QCoreApplication::translate("CaptureLauncher", "Take new screenshot", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CaptureLauncher: public Ui_CaptureLauncher {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CAPTURELAUNCHER_H
