// SPDX-License-Identifier: GPL-3.0-or-later
// SPDX-FileCopyrightText: 2017-2019 Alejandro Sirgo Rica & Contributors

#pragma once

class QString;

namespace GlobalValues {

int buttonBaseSize();
QString versionInfo();
QString iconPath();
QString iconPathPNG();
QString trayIconPath();
}
