/****************************************************************************
** Meta object code from reading C++ file 'texttool.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../flameshot-14.0.0/src/tools/text/texttool.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'texttool.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8TextToolE_t {};
} // unnamed namespace

template <> constexpr inline auto TextTool::qt_create_metaobjectdata<qt_meta_tag_ZN8TextToolE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextTool",
        "drawEnd",
        "",
        "QPoint",
        "point",
        "drawMove",
        "drawStart",
        "CaptureContext",
        "context",
        "pressed",
        "CaptureContext&",
        "onColorChanged",
        "QColor",
        "color",
        "onSizeChanged",
        "size",
        "updateText",
        "string",
        "updateFamily",
        "updateFontUnderline",
        "underlined",
        "updateFontStrikeOut",
        "strikeout",
        "updateFontWeight",
        "QFont::Weight",
        "weight",
        "updateFontItalic",
        "italic",
        "updateAlignment",
        "Qt::AlignmentFlag",
        "alignment"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'drawEnd'
        QtMocHelpers::SlotData<void(const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'drawMove'
        QtMocHelpers::SlotData<void(const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'drawStart'
        QtMocHelpers::SlotData<void(const CaptureContext &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'pressed'
        QtMocHelpers::SlotData<void(CaptureContext &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 8 },
        }}),
        // Slot 'onColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'onSizeChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'size'
        QtMocHelpers::SlotData<int() const>(15, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'updateText'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'updateFamily'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'updateFontUnderline'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'updateFontStrikeOut'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'updateFontWeight'
        QtMocHelpers::SlotData<void(QFont::Weight)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'updateFontItalic'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'updateAlignment'
        QtMocHelpers::SlotData<void(Qt::AlignmentFlag)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextTool, qt_meta_tag_ZN8TextToolE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextTool::staticMetaObject = { {
    QMetaObject::SuperData::link<CaptureTool::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TextToolE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TextToolE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8TextToolE_t>.metaTypes,
    nullptr
} };

void TextTool::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextTool *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->drawEnd((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->drawMove((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->drawStart((*reinterpret_cast<std::add_pointer_t<CaptureContext>>(_a[1]))); break;
        case 3: _t->pressed((*reinterpret_cast<std::add_pointer_t<CaptureContext&>>(_a[1]))); break;
        case 4: _t->onColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->onSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: { int _r = _t->size();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->updateText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->updateFamily((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->updateFontUnderline((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->updateFontStrikeOut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->updateFontWeight((*reinterpret_cast<std::add_pointer_t<QFont::Weight>>(_a[1]))); break;
        case 12: _t->updateFontItalic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->updateAlignment((*reinterpret_cast<std::add_pointer_t<Qt::AlignmentFlag>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *TextTool::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextTool::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TextToolE_t>.strings))
        return static_cast<void*>(this);
    return CaptureTool::qt_metacast(_clname);
}

int TextTool::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CaptureTool::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
