/****************************************************************************
** Meta object code from reading C++ file 'capturetool.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../flameshot-14.0.0/src/tools/capturetool.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'capturetool.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11CaptureToolE_t {};
} // unnamed namespace

template <> constexpr inline auto CaptureTool::qt_create_metaobjectdata<qt_meta_tag_ZN11CaptureToolE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CaptureTool",
        "requestAction",
        "",
        "Request",
        "r",
        "drawEnd",
        "QPoint",
        "p",
        "drawMove",
        "drawMoveWithAdjustment",
        "drawStart",
        "CaptureContext",
        "context",
        "pressed",
        "CaptureContext&",
        "onColorChanged",
        "QColor",
        "c",
        "onSizeChanged",
        "size",
        "Type",
        "NONE",
        "TYPE_PENCIL",
        "TYPE_DRAWER",
        "TYPE_ARROW",
        "TYPE_SELECTION",
        "TYPE_RECTANGLE",
        "TYPE_CIRCLE",
        "TYPE_MARKER",
        "TYPE_MOVESELECTION",
        "TYPE_UNDO",
        "TYPE_COPY",
        "TYPE_SAVE",
        "TYPE_EXIT",
        "TYPE_OPEN_APP",
        "TYPE_PIXELATE",
        "TYPE_REDO",
        "TYPE_PIN",
        "TYPE_TEXT",
        "TYPE_CIRCLECOUNT",
        "TYPE_SIZEINCREASE",
        "TYPE_SIZEDECREASE",
        "TYPE_INVERT",
        "TYPE_ACCEPT",
        "TYPE_CANCEL"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestAction'
        QtMocHelpers::SignalData<void(enum Request)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'drawEnd'
        QtMocHelpers::SlotData<void(const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'drawMove'
        QtMocHelpers::SlotData<void(const QPoint &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'drawMoveWithAdjustment'
        QtMocHelpers::SlotData<void(const QPoint &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'drawStart'
        QtMocHelpers::SlotData<void(const CaptureContext &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'pressed'
        QtMocHelpers::SlotData<void(CaptureContext &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 12 },
        }}),
        // Slot 'onColorChanged'
        QtMocHelpers::SlotData<void(const QColor &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'onSizeChanged'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'size'
        QtMocHelpers::SlotData<int() const>(19, 2, QMC::AccessPublic, QMetaType::Int),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(20, 20, QMC::EnumFlags{}).add({
            {   21, Type::NONE },
            {   22, Type::TYPE_PENCIL },
            {   23, Type::TYPE_DRAWER },
            {   24, Type::TYPE_ARROW },
            {   25, Type::TYPE_SELECTION },
            {   26, Type::TYPE_RECTANGLE },
            {   27, Type::TYPE_CIRCLE },
            {   28, Type::TYPE_MARKER },
            {   29, Type::TYPE_MOVESELECTION },
            {   30, Type::TYPE_UNDO },
            {   31, Type::TYPE_COPY },
            {   32, Type::TYPE_SAVE },
            {   33, Type::TYPE_EXIT },
            {   34, Type::TYPE_OPEN_APP },
            {   35, Type::TYPE_PIXELATE },
            {   36, Type::TYPE_REDO },
            {   37, Type::TYPE_PIN },
            {   38, Type::TYPE_TEXT },
            {   39, Type::TYPE_CIRCLECOUNT },
            {   40, Type::TYPE_SIZEINCREASE },
            {   41, Type::TYPE_SIZEDECREASE },
            {   42, Type::TYPE_INVERT },
            {   43, Type::TYPE_ACCEPT },
            {   44, Type::TYPE_CANCEL },
        }),
    };
    return QtMocHelpers::metaObjectData<CaptureTool, qt_meta_tag_ZN11CaptureToolE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CaptureTool::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CaptureToolE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CaptureToolE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CaptureToolE_t>.metaTypes,
    nullptr
} };

void CaptureTool::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CaptureTool *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestAction((*reinterpret_cast<std::add_pointer_t<enum Request>>(_a[1]))); break;
        case 1: _t->drawEnd((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->drawMove((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 3: _t->drawMoveWithAdjustment((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->drawStart((*reinterpret_cast<std::add_pointer_t<CaptureContext>>(_a[1]))); break;
        case 5: _t->pressed((*reinterpret_cast<std::add_pointer_t<CaptureContext&>>(_a[1]))); break;
        case 6: _t->onColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->onSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: { int _r = _t->size();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CaptureTool::*)(Request )>(_a, &CaptureTool::requestAction, 0))
            return;
    }
}

const QMetaObject *CaptureTool::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CaptureTool::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CaptureToolE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CaptureTool::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void CaptureTool::requestAction(Request _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
